import { world, system, MolangVariableMap, Player } from "@minecraft/server";
import { ActionFormData, ModalFormData } from "@minecraft/server-ui";

import * as minecraft from "@minecraft/server";
import * as ui from "@minecraft/server-ui";

Player.prototype.cooldown_timer = 0;
Player.prototype.special_timer = 0;

world.afterEvents.dataDrivenEntityTrigger.subscribe((data) => {
  const { eventId: event, entity } = data;
  if (entity.typeId == "bf:npc") {
    const playerInteractor = entity.dimension.getPlayers({
      location: entity.location,
      tags: ["interactWith"],
      maxDistance: 8,
      closest: 1,
    })[0];

    if (event == "interact") {
      if (playerInteractor) {
        if (!playerInteractor.hasTag("talked_npc")) {
          MenuNPC({ player: playerInteractor, entity: entity });
          playerInteractor.removeTag("interactWith");
          playerInteractor.removeTag("openMenu");
        }
      }
    }
    if (event == "interact2") {
      if (playerInteractor) {
        if (playerInteractor.hasTag("talked_npc")) {
          MenuNPC5({ player: playerInteractor, entity: entity });
          playerInteractor.removeTag("interactWith");
          playerInteractor.removeTag("openMenu2");
        }
      }
    }
  } else {
    return;
  }
});

function MenuNPC({ player, entity }) {
  const form = new ActionFormData();
  form.title(`${entity.id}`);
  form.body(
    `§lScientific§r§7\n\nHello player, I see you're lost, well, don't worry, I'll help you by telling you how things work around here.`
  );
  let functions = [];
  {
    form.button("Ok");
    functions.push((res) => {
      MenuNPC1({ player, entity });
    });
  }

  form.show(player).then((res) => {
    const canceled = res.canceled;
    if (canceled == true) {
      MenuNPC({ player, entity });
      return;
    }
    functions[res.selection]();
  });
}

function MenuNPC1({ player, entity }) {
  const form = new ActionFormData();
  form.title(`${entity.id}`);
  form.body(
    `§lScientific§r§7\n\nTo begin with, you can look for valuable minerals, which are essential for creating weapons and armor. Here's a book to explain how this world works.`
  );
  let functions = [];
  {
    form.button("Take");
    functions.push((res) => {
      MenuNPC2({ player, entity });
      if (!player.hasTag("has_book")) {
        player.runCommand("give @s bf:book");
        player.addTag("has_book");
      } else {
        player.runCommand(
          `tellraw @s {"rawtext":[{"text":"§cYou already received the book!"}]}`
        );
      }
    });
  }

  form.show(player).then((res) => {
    const canceled = res.canceled;
    if (canceled == true) {
      MenuNPC1({ player, entity });
      return;
    }
    functions[res.selection]();
  });
}

function MenuNPC2({ player, entity }) {
  const form = new ActionFormData();
  form.title(`${entity.id}`);
  form.body(
    `§lScientific§r§7\n\nSome weapons you can use attacks that take a while to charge, but they are stronger. To use weapon skills, use [Interact] or hold it.`
  );
  let functions = [];
  {
    form.button("Alright");
    functions.push((res) => {
      MenuNPC3({ player, entity });
    });
  }

  form.show(player).then((res) => {
    const canceled = res.canceled;
    if (canceled == true) {
      MenuNPC2({ player, entity });
      return;
    }
    functions[res.selection]();
  });
}

function MenuNPC3({ player, entity }) {
  const form = new ActionFormData();
  form.title(`${entity.id}`);
  form.body(
    `§lScientific§r§7\n\nIf you lose your book, come back and talk to me! I can give you another one. Inside this book you'll find out how to craft the new weapons and how to use them!`
  );
  let functions = [];
  {
    form.button("Okay");
    functions.push((res) => {
      MenuNPC4({ player, entity });
    });
  }

  form.show(player).then((res) => {
    const canceled = res.canceled;
    if (canceled == true) {
      MenuNPC3({ player, entity });
      return;
    }
    functions[res.selection]();
  });
}

function MenuNPC4({ player, entity }) {
  const form = new ActionFormData();
  form.title(`${entity.id}`);
  form.body(
    `§lScientific§r§7\n\nI think I've told you enough hero, if you need to come back and talk to me, also if you've lost the book I'll be there! Have a good adventure hero!`
  );
  let functions = [];
  {
    form.button("Thanks");
    functions.push((res) => {
      player.addTag("talked_npc");
    });
  }

  form.show(player).then((res) => {
    const canceled = res.canceled;
    if (canceled == true) {
      MenuNPC4({ player, entity });
      return;
    }
    functions[res.selection]();
  });
}

function MenuNPC5({ player, entity }) {
  const form = new ActionFormData();
  form.title(`${entity.id}`);
  form.body(`§lScientific§r§7\n\nHey Hero! What would you like to do?`);
  let functions = [];
  {
    form.button("Repeat");
    functions.push((res) => {
      player.removeTag("talked_npc");
      MenuNPC({ player, entity });
    });
  }
  {
    form.button("Book");
    functions.push((res) => {
      player.runCommand("give @s bf:book");
    });
  }

  form.show(player).then((res) => {
    const canceled = res.canceled;
    if (canceled == true) {
      return;
    }
    functions[res.selection]();
  });
}

export function getDistance(playerLocationA, playerLocationB) {
  const { x: x1, y: y1, z: z1 } = playerLocationA;
  const { x: x2, y: y2, z: z2 } = playerLocationB;
  const distance = Math.sqrt((x2 - x1) ** 2 + (y2 - y1) ** 2 + (z2 - z1) ** 2);
  return Math.floor(distance);
}


system.runInterval(() => {
  const players = world.getAllPlayers();
  for (let player of players) {
    let entities = world.getDimension("overworld").getEntities({
      type: "bf:npc",
      location: {
        x: player.location.x,
        y: player.location.y,
        z: player.location.z,
      },
      closest: 1,
    });

    for (let entitie of entities) {
      const distance = getDistance(player.location, entitie.location);

      let distanceTextToDisplay = "";
      distanceTextToDisplay += `§f - §a ${distance}m`;

      distanceTextToDisplay.slice(0, -2);

      if (!player.hasTag("talked_npc")) {
        player.onScreenDisplay.setActionBar(
          `§b[!]§e Talk to the npc inside the base! ${distanceTextToDisplay}`
        );
      }

    }}})